#!/usr/bin/env python

import serial
import threading
import time

from envbox_database import *
from envbox_parser import *

"""*****************************************************"""
"""                                                     """
"""		Software Defines & Parmeters		"""
"""                                                     """
"""*****************************************************"""
SOFTWARE_VERSION = '1.0'

RS232_BAUDRATE = 115200

SERVER_TCP_IP = '192.168.1.1'
SERVER_TCP_PORT = 3081 
SERVER_BUFFER_SIZE = 128
SERVER_TIMEOUT = 600

RETRIES = 3 

class envbox (threading.Thread):

   def __init__( self, device ):
     threading.Thread.__init__(self)

     self.__devname = device
     self.__timeout = 0.0001 

     self . dev 		= serial.Serial( device, RS232_BAUDRATE, timeout=0.0001 )
     self . dev.baudrate	= RS232_BAUDRATE
     self . dev.bytesize	= serial.EIGHTBITS	#number of bits per bytes
     self . dev.parity		= serial.PARITY_NONE	#set parity check: no parity
     self . dev.stopbits	= serial.STOPBITS_ONE	#number of stop bits
     self . dev.timeout		= 0.0001			#non-block read
     self . dev.xonxoff		= False			#disable software flow control
     self . dev.rtscts		= False			#disable hardware (RTS/CTS) flow control
     self . dev.dsrdtr		= False			#disable hardware (DSR/DTR) flow control
     self . dev.writeTimeout	= 2			#timeout for write
     #self.flush()    
     #self.start() 


   def writeb( self, bytea ):
     self . flush()
     self . dev . write( ''.join( map(chr, bytea) ) )


   def readb( self ):
     res = self.dev.read()
     return res
     #res = self.dev.readline()
     #return [ord(c) for c in res] 


   def flush( self ):
     try:
       flush = self.readb()
     except:
       pass


   def cmd( self, name, value=None ):
     cmd = envbox_db . getCommandByName( name )
     if cmd is None:
       return None

     val = None
     if value:   
       val = envbox_db . getValueByName( name, value )

     try:
       command = envbox_command_builder . createCommand( cmd, val )
       self.writeb( command )
     except:
       print( 'Failed to send command to envbox' )
       return None

     response = ''
     rsize = 666
     for i in range( 1, 500 ):
       try:
         response = response + self.dev.read()
         #time.sleep( 1 )
	 #print response
	 while len( response ) > 0:
	   if ord(response[0]) == 255:
	     break
	   elif ord(response[0]) == 254:
	     break
	   else:
	     response = response[ 1: ]
       except:
         print( 'Error listening response' )
         pass
       if( len( response ) >= 8 ):
         rsize = int(ord(response[ 7 ]))
         #print rsize
       if( len( response ) >= 8 + rsize + 1 ):
         break

     #print [ord(c) for c in response]
     try:
     	retval = envbox_command_builder . parseCommand( [ord(c) for c in response] )
	return envbox_db . getValueName( name, retval )
     except:
         print( 'Error while reading response' )
         print response
         return None



   def close( self ):
     self.dev.close()
     try:
        self.dev=serial.Serial( self.__devname, RS232_BAUDRATE, timeout=0.5 )
     except:
	print( 'Failed to re-open RS232 Port' )




   def run( self ):
     while True:
        rsize = 666
        response = '\xFF'

        # Wait for sync header
        reschar = 'a'
        while not ord(reschar) == 255:
           try:
              reschar = self.dev.read()
              if len(reschar) == 0:
                 reschar = 'a'
           except:
              reschar = 'a'
              pass

        while True: 
           try:
              response = response + self . dev . read()
              if( len(response)==4 ):
                 rsize = int(ord(response[ 3 ]))
                 #print 'Size = ' + str(rsize)
              if( len(response)== 6 + rsize ):
                 print [ord(c) for c in response]
                 break
           except:
              print 'error'
              pass
        print response
